#!/usr/bin/perl
# ----------------------------------------------------------------------
# | Module:     file_io.pl                                             |
# |                                                                    |
# | Purpose:    Test a few file I/O functions                          |
# |                                                                    |
# |                                                                    |
# |                                                                    |
# ----------------------------------------------------------------------


# --------------------------------------------------------------
# | The stuff necessary to use the Wine/Perl interface         |
# --------------------------------------------------------------
use lib "wine/blib/arch";
use lib "wine/blib/lib";
use ExtUtils::testlib;
use wine;
use wine_util;



    # ----------------------------------------------------------------------
    # | Entry point (if lbpi is used)                                      |
    # |                                                                    |
    # |                                                                    |
    # |                                                                    |
    # ----------------------------------------------------------------------
    sub main
    {
        $wine = wine->new;
        $wine->init ("wine_callback");
    }



    # ----------------------------------------------------------------------
    # | Subroutine:  wine_callback                                         |
    # |                                                                    |
    # | Purpose:     Make various file I/O wine API calls                  |
    # |                                                                    |
    # | Usage:       wine_callback                                         |
    # |                                                                    |
    # | Returns:     (none)                                                |
    # ----------------------------------------------------------------------
    sub wine_callback
    {
        # Locals
        my  $fname = "bark.dog";
        my  $fh;
        my  ($str, $n, $t);

        my  ($GENERIC_READ, $GENERIC_WRITE) = (0x80000000, 0x40000000);
        my  ($FILE_BEGIN)                   = (0);
    
    # Begin sub wine_callback
        
        # --------------------------------------------------------------
        # | Initializations                                            |
        # --------------------------------------------------------------
        $wine->ret_type ("int");
        $wine->module   ("kernel32");

        
        # --------------------------------------------------------------
        # | Create file                                                |
        # --------------------------------------------------------------
        $fh = $wine->CreateFileA
        (
            $fname,
            ($GENERIC_READ|$GENERIC_WRITE),
            0,
            0,
            1,
            0x80,
            0
        );
        if ($fh >= 0)
        {
        
            # --------------------------------------------------------------
            # | Write some data to it                                      |
            # --------------------------------------------------------------
            $str = "dogblaster";
            $n = 0;
            $r = $wine->WriteFile
            (
                $fh,
                $str,
                length ($str),
                \$n,
                0
            );
            assert ($r == 1);
            assert ($n == length ($str));
        
            # --------------------------------------------------------------
            # | Seek to beginning                                          |
            # --------------------------------------------------------------
            $r = $wine->SetFilePointer 
            (
                $fh,
                0,
                0,
                $FILE_BEGIN
            );
            assert ($r == $FILE_BEGIN);
        
            # --------------------------------------------------------------
            # | Read data back again                                       |
            # --------------------------------------------------------------
            $t = "." x 20;
            $n = 0;
            $r = $wine->ReadFile 
            (
                $fh,
                \$t,
                length ($str),
                \$n,
                0
            );
            assert ($r == 1);
            assert ($n == length ($str));
            assert (substr ($t, 0, length ($str)) eq $str);
        
            # --------------------------------------------------------------
            # | Close handle                                               |
            # --------------------------------------------------------------
            $r = $wine->CloseHandle ($fh);
            assert ($r == 1);
        
            # --------------------------------------------------------------
            # | Delete file                                                |
            # --------------------------------------------------------------
            $r = $wine->DeleteFileA ($fname);
            assert ($r == 1);

        }  # end if (CreateFile succeeds)

    } # End sub wine_callback



    # ----------------------------------------------------------------------
    # | Allow for script to be called on its own                           |
    # |                                                                    |
    # |                                                                    |
    # |                                                                    |
    # ----------------------------------------------------------------------
    main ();
