# --------------------------------------------------------------------------------
# | Module:      wine.pm                                                         |
# | ---------------------------------------------------------------------------- |
# | Purpose:     Module to supply wrapper around and support for gateway to wine |
# |              API functions                                                   |
# |                                                                              |
# | Methods:                                                                     |
# |                                                                              |
# |     new  -- object constructor                                               |
# |     init -- initialize wine                                                  |
# |     call -- call wine API function                                           |
# |                                                                              |
# |     There are also object accessor function implemented with AUTOLOAD        |
# --------------------------------------------------------------------------------



# ------------------------------------------------------------------------
# | [This is generated by h2xs]                                          |
# |                                                                      |
# |                                                                      |
# ------------------------------------------------------------------------
package wine;

    use strict;
    use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $AUTOLOAD);

    require Exporter;
    require DynaLoader;
    use AutoLoader;

    @ISA = qw(Exporter DynaLoader);

    # Items to export into callers namespace by default. Note: do not export
    # names by default without a very good reason. Use EXPORT_OK instead.
    # Do not simply export all your public functions/methods/constants.
    @EXPORT = qw(
        new
        init
        call
        err
        AUTOLOAD
    );

$VERSION = '0.01';
bootstrap wine $VERSION;



# Preloaded methods go here.

# ------------------------------------------------------------------------
# | Method:    new                                                       |
# | -------------------------------------------------------------------- |
# | Purpose:   Object constructor                                        |
# |                                                                      |
# | Usage:     $obj->new                                                 |
# |                                                                      |
# | Returns:   new wine object                                           |
# ------------------------------------------------------------------------
sub AUTOLOAD
{
    # ----------------------------------------------
    # | Locals                                     |
    # ----------------------------------------------
    my  ($pkg, $func);
    my  ($self, $copy);

    my  $chain_table =
    [
        # --------------------------------------
        # | function      key          value   |
        # --------------------------------------
        [   'kernel32',   'module',    undef   ],
        [   'user32',     'module',    undef   ],
        [   'int',        'ret_type',  undef   ],
        [   'ptr',        'ret_type',  undef   ],
        [   'debug_on',   'debug',         1   ],
        [   'debug_off',  'debug',         0   ],
    ];

    my  $t;

# Begin AUTOLOAD

    # --------------------------------------------------------------
    # | Figure out who we are                                      |
    # --------------------------------------------------------------
    ($pkg, $func) = (split /::/, $AUTOLOAD)[0,1];

    # --------------------------------------------------------------
    # | Any function that is in the @EXPORT array is passed thru   |
    # | to AutoLoader to pick up the appropriate XS extension      |
    # --------------------------------------------------------------
    if (grep ($_ eq $func, @EXPORT))
    {
        $AutoLoader::AUTOLOAD = $AUTOLOAD;
        goto &AutoLoader::AUTOLOAD;
    }

    # --------------------------------------------------------------
    # | Ignore this                                                |
    # --------------------------------------------------------------
    return
        if ($func eq 'DESTROY');

    # --------------------------------------------------------------
    # | Grab object ref                                            |
    # --------------------------------------------------------------
    $self = shift;

    # --------------------------------------------------------------
    # | Chain modifier function                                    |
    # --------------------------------------------------------------
    if (($t) = grep ($_->[0] eq $func, @{$chain_table}))
    {

        # --------------------------------------------------------------
        # | Chain modifiers do not modify the object itself, so we     |
        # | need to make a copy                                        |
        # --------------------------------------------------------------
        $copy = { %{$self} };
        bless $copy, $pkg;

        # --------------------------------------------------------------
        # | Now, set the temporary value in the copy, as directed by   |
        # | the table (a value of 'undef' means set the value to the   |
        # | function name)                                             |
        # --------------------------------------------------------------
        $copy->{$t->[1]} =
        (
            defined ($t->[2]) ?
            $t->[2]           :
            $func
        );

        # --------------------------------------------------------------
        # | Return copy                                                |
        # --------------------------------------------------------------
        return ($copy);

    }  # end if (chain modifier function)

    # --------------------------------------------------------------
    # | Accessor function                                          |
    # --------------------------------------------------------------
    if (grep ($_ eq $func, keys %{$self}))
    {
        (@_ > 0) and ($self->{$func} = shift);
        return ($self);
    }

    # --------------------------------------------------------------
    # | Otherwise, assume any undefined method is the name of a    |
    # | wine API call, and all the args are to be passed through   |
    # --------------------------------------------------------------
    else
    {
        $self->call
        (
            function => $func,
            args     => [ @_ ]
        );
    }

} # End AUTOLOAD



# Autoload methods go after =cut, and are processed by the autosplit program.
1;
__END__



# ------------------------------------------------------------------------
# | pod documentation                                                    |
# |                                                                      |
# |                                                                      |
# ------------------------------------------------------------------------

=head1 NAME

wine - Perl extension for calling wine API functions

=head1 SYNOPSIS

    use wine;

    $wine = wine->new;                      # Create wine object
    $wine->init ("run_wine_tests");         # Initialize wine

    sub run_wine_tests
    {
        $wine->GlobalAddAtomA ("foobar");   # Call API function
    }



=head1 DESCRIPTION

This module provides a gateway for calling wine API functions from
a Perl script.

Some setup is necessary before code like that in the B<SYNOPSIS>
above will work; See the section titled GETTING STARTED below.

In addition, there are a number of syntactically different
ways of calling a wine API function, also documented below.



=head1 GETTING STARTED

It would be nice if you could just create a Perl script, put
in calls to the methods provided by the wine extension,
and run the script.  And you almost can ... but not quite.  

The reasons are a bit esoteric, but the long and short of it
is this:  There are symbols defined by both the wine libraries
and the standard C library.  In order for wine to work properly,
the libraries have to be loaded in the right order, so that the
right symbols are picked up (specifically, the wine libraries have
to be loaded first).  When the Perl interpreter is run directly,
that doesn't happen because the standard C library is loaded when
Perl starts up, and the wine libraries aren't loaded until later,
at run-time.

Generally, there are two ways around this problem:  [1] Use the
B<LD_PRELOAD> environment variable to control the order in which
libraries are loaded, and [2] Use the B<lbpi> utility provided
with this package.  Each method is outlined below.  (Sorry about
all the gory detail, but if you wade through it just once and
pick a method, you can pretty much forget about it after that).



=head1 USING LD_PRELOAD

This is by far the simplest method, and it is the preferred method if
it works.  The wine library that seems to matter is libkernel32.so,
so simply set B<LD_PRELOAD> to point to it, by whatever means are
appropriate for the shell you're using.  For example, in ksh and similar
shells:

    export LD_PRELOAD=libkernel32.so

This forces the wine library to be loaded first. You should
then be able to run Perl scripts (like like the one
coded in the B<SYNOPSIS> above, or any of the sample scripts)
directly from the command line, and things should work.

Be aware that setting B<LD_PRELOAD> is a potential source of
other problems.  Sure, it allows Perl scripts which call into
the wine API to work.  But most things aren't Perl scripts which
call wine.  Many executables will fail with B<LD_PRELOAD> set
as above (because they really *do* want the standard C library
symbols, and *not* those defined by wine).

So if you're going to set B<LD_PRELOAD>, you probably want to
set it only temporarily, just for the duration of your testing.
(I can't tell you how many times I've gone to compile something and
had gcc dump core, because I exported B<LD_PRELOAD> in a terminal
window and then forgot and tried to compile in the same window ...)



=head1 USING lbpi

The alternative to using B<LD_PRELOAD> is to use a utility provided
by this package called B<lbpi> (little-bitty Perl interpreter),
which gets around the load-order problem in a different manner.

B<lbpi> works because, unlike the Perl interpreter, it loads
both the standard C library and the wine libraries when it starts
(instead of loading the wine libraries later, at run-time), and it
loads them in the proper order.  It then embeds a Perl interpreter,
compiles the script you pass it on the command-line, and calls into
the script using an entry point you specify on the command-line.

Because of the sequence described above, the use of B<lbpi>
requires a little extra code in your Perl script.  B<lbpi>
has to call into your script at some entry point subroutine,
which means any mainline code in the script never gets
executed.  So you can't just create a Perl script like this:

    use wine;

    $wine = wine->new;                      # Create wine object
    $wine->init ("run_wine_tests");         # Initialize wine

    sub run_wine_tests
    {
        [...]
    }

because the calls to wine->new() and $wine->init() are mainline
code, and won't ever be run.  Instead, the code that would
normally be mainline needs to be put into a subroutine, as
follows:

    use wine;

    sub main
    {
        $wine = wine->new;                      # Create wine object
        $wine->init ("run_wine_tests");         # Initialize wine
    }

    sub run_wine_tests
    {
        [...]
    }

Then, when you invoke B<lbpi>, you tell it the name of the Perl
script to load, and the name of the function which is meant to
be the entry point.  B<lbpi> will compile the script, and then
call the entry point subroutine.  In the example above, if B<lbpi>
is told that main() is the entry point, then the calls to
wine->new() and $wine->init() will be executed.

There are two ways to run a Perl script with B<lbpi>:

=head2 [1] Run lbpi directly from the command-line:

B<lbpi> can be run directly from the command-line as follows:

    lbpi --perl-script <script> [--entry-sub <entry_point>]

B<--perl-script> specifies the name of the script to run.
Alternately, the name of the script may be specified without
using the B<--perl-script> option, as the first argument on
the command-line following all the options.

B<--entry-sub> specifies the name of the subroutine in the script
which B<lbpi> calls into, after the script has been loaded.  If the
subroutine is not defined in the script, B<lbpi> (not surprisingly)
will fail.  The default entry sub is main(), so if your script has
a subroutine called main, B<--entry-sub> may be omitted.

For example, consider the following Perl script B<foo.pl>:

    #!/usr/local/bin/perl

    use wine;

    print "I will not be displayed\n";  # won't be executed
    dump;                               # won't be executed

    sub main
    {
        # [do some wine stuff]
    }

Because the intended entry point is main() (the default), all
of the following are equivalent command-lines to run B<foo.pl>
using B<lbpi>:

    lbpi --perl-script=foo.pl --entry-sub=main
    lbpi --perl-script=foo.pl
    lbpi --entry-sub=main foo.pl
    lbpi foo.pl

Note that the two statements

    print "I will not be displayed\n";  # won't be executed
    dump;                               # won't be executed

will never be executed by B<lbpi>.  They will be compiled
by the embedded Perl interpreter, but never run.  The line

    use wine;

is also only seen at compile time, and not at run-time, but
the 'use' directive is a compile-time directive, so that is
all right.  It can be outside of subroutine main().

=head2 [2] Run lbpi via a 'shebang' line:

B<lbpi> can be run directly from the Perl script using the
B<#!> 'shebang' notation, if that notation is supported by
the shell you use.  To use this
method, the above B<foo.pl> would be written like this:

    #!lbpi
    #!perl

    use wine;

    sub main
    {
        # [do some wine stuff]
    }

With this construct, you can simply run

    foo.pl

directly from the command line.  B<Both> shebang lines (B<#!lbpi>
and B<#!perl>, in that order) are necessary.  What actually
happens is this:  When you run B<foo.pl>, the first shebang line
causes B<lbpi> to be run, with B<foo.pl> passed as an argument;
the rest of the script is ignored (for the moment).  Within
B<lbpi>, the embedded Perl interpreter loads B<foo.pl> again, in
a way which causes the first shebang line to be ignored and
compilation to begin starting with the B<#!perl> line.  The
embedded Perl interpreter compiles the remainder of B<foo.pl>
like usual and then calls main().

There is one hitch to this method:  The entry sub B<must> be
main().  There is no way to specify an alternate entry point
subroutine on the shebang line.  Sue me.



=head1 WINE INITIALIZATION

Okay.  Now that we're through that mucky-muck, it's time to get
on to the subject of calling wine API functions.  From this point
on, all examples will assume the use of the B<LD_PRELOAD> method
described above.  (The sample scripts are written such that they
can be run either with lbpi, or directly using B<LD_PRELOAD>).

The first step in accessing wine is to create a B<wine object>, using
the new() method, as follows:

    $wine = wine->new;

Before you can use the wine object to call wine API functions from
your script, wine has to be initialized.  In the C world, the wine
initialization routine is written such that the function that calls
it never regains control; control is passed to a callback function
instead.

Accordingly, the Perl interface must behave similarly.  Once you've
created a wine object, you tell it to initialize wine using the
init() method:

    $wine->init ("dogger");

The argument to init() is the callback -- the Perl subroutine
which will receive control after wine has been initialized (dogger(),
in this case).  If it is omitted, wine_callback() is assumed.
If you have not defined the specified subroutine in your script,
all is lost.  (Well, it isn't lost; you'll know right where it is.
But it won't work ...)

Notice that the init() method never returns.  Any statements
immediately following the init() call will not be executed;
control is transferred to the specified callback instead.

The callback, then, is where all the interesting stuff happens.
That's where you can use the wine object you've created and
initialized to call wine API functions, passing them parameters
that you create in Perl variables, and inspecting the return
values to see if they're what you expect.



=head1 CALLING WINE API FUNCTIONS

To call a wine API function, you need to specify four things:

=over 4

=item B<module>

The wine module (DLL) in which the function to be called resides

=item B<function>

The name of the function to be called

=item B<function args>

The arguments to be passed to the function

=item B<return type>

The type of value the function returns (either a pointer or an
integer).  This is necessary in order for the value to be converted
properly in being passed from C back to Perl.

=back

The most direct way to call an API function is to use the wine object's
call() method, which is documented below.  In addition, this package
provides a number of helper methods and other syntactic sugar which
make calling API functions a little less cumbersome.  Those methods
are documented further below.



=head1 CALLING API FUNCTIONS:  THE BRUTE-FORCE WAY

Using the call() method directly provides the greatest flexibility,
but uses a very Perl-ish syntax that may seem a bit cumbersome
if you're not used to it.  If this seems ugly, see further below
for some alternatives.

To use the call() method directly, the items listed above are
passed as a hash initializer with the following keys:  B<module,
function, args, ret_type>.  The valid values for B<ret_type> are
"int" and "ptr".  If only one argument is to be passed through
to the API function, it may be specified as a single value for
the B<args> key; if more than one argument is passed, the value
for the B<args> key must be an anonymous array constructor which
groups the arguments.

In addition, there is one other valid key value:  B<debug>.  If
non-zero, it causes trace information to be dumped to stdout as
control winds it way through Perl to C, into wine, and back.



=head1 EXAMPLES (BRUTE-FORCE WAY)

Here's a complete script which calls the wine GlobalAddAtomA() API
function.  It assumes the use of the B<LD_PRELOAD> environment
variable, uses the default wine initialization callback
wine_callback(), and uses the call() method directly, as described
above.  It prints the integer value returned from GlobalAddAtomA().

    $wine = wine->new;
    $wine->init;

    sub wine_callback
    {
        $atom = $wine->call
        (
            module   => "kernel32",
            function => "GlobalAddAtomA"
            args     => [ "dogblaster" ],
            ret_type => "int",
            debug    => 0
        );
        print "GlobalAddAtomA() returned [$atom]\n";
    }

Recall what happens here: A wine object is created, and then wine
is initialized, which (implicitly) calls back to wine_callback().
In wine_callback(), GlobalAddAtomA() is called, which adds an
atom named "dogblaster" and returns its numeric ID.

Because only one argument was passed to GlobalAddAtomA() above,
the args initializer could have been specified without the anonymous
array constructor, like this:

    $atom = $wine->call
    (
        module   => "kernel32",
        function => "GlobalAddAtomA"
        args     => "dogblaster",
        ret_type => "int",
        debug    => 0
    );

Here is a call to the GlobalGetAtomNameA() API function, which takes
a previously created atom ID as a parameter, and stashes the atom's
name in a return buffer (the buffer is passed as the second parameter,
the length of the buffer the third).

    $buf = "." x 20;
    $len = $wine->call
    (
        module   => "kernel32",
        function => "GlobalGetAtomNameA"
        args     => [ $atom, \$buf, length ($buf) ],
        ret_type => "int",
        debug    => 0
    );

A few comments:

=over 4

=item

[1] Here, we are passing 3 parameters to the wine API function,
so the value of the B<args> key must be grouped by an anonymous
array constructor.

[2] GlobalGetAtomNameA() does not allocate memory; it just copies
the atom name into the memory pointed to by its first parameter,
which it assumes to have already been allocated (as C functions
are wont to do).  Thus, the Perl script must make variable $buf
point to some memory before passing it to wine.  Here, we make $buf
point to a buffer of 20 bytes (probably plus one more for a null
terminator); the third parameter passed to GlobalGetAtomNameA()
is the length, which ensures the API function won't overwrite
the buffer.

[3] The call() method understands passing by reference, just like
any other Perl method or sub.  Here, we specified

    \$buf

as the second parameter to pass to GlobalGetAtomNameA(), so that
when it returns, variable $buf in the Perl script has actually been
modified.  We could have done this:

    args => [ $atom, $buf, length ($buf) ],
        
which would be fine as far as GlobalGetAtomNameA() is concerned.
But when the call was finished, $buf would be unchanged.  The
wine/Perl gateway code takes care of resolving references and
causing an appropriate reference to be passed to the wine API
function.

=back



=head1 WINE OBJECT ACCESSOR FUNCTIONS

Frequently, you find yourself calling several API functions in
a row, and many of the key values are the same from one call to
the next.  A wine object has a set of accessor functions called
B<module>, B<function>, B<ret_type>, B<args>, and B<debug> (the
same as the $wine->call() key values), which may be used to set
default values for the object.  Thus, any values that will not
change from one call to the next can be set once at the start and
omitted when using the call() method, and the default values will
be supplied.


For instance, calling several API functions from the same module:

    $wine->module ("kernel32");
    $wine->ret_type ("int");
    $wine->call (function => "GlobalAddAtomA",     args => [ ... ]);
    $wine->call (function => "GlobalGetAtomNameA", args => [ ... ]);
    $wine->call (function => "GlobalDeleteAtomA",  args => [ ... ]);

Or several calls in a row to the same API function:

    $wine->module ("kernel32");
    $wine->function ("GlobalAddAtomA");
    $wine->ret_type ("int");
    $wine->debug (1);

    $wine->call (args => "foo");
    $wine->call (args => "bar");
    $wine->call (args => "xyz");



=head1 SYNTACTIC SUGAR

The accessor functions mentioned above help to keep individual
calls to the wine API from getting quite so unwieldy.  In addition,
a couple items of syntactic sugar are provided to clean things up
a bit more.


=head2 AUTOLOAD methods

The module uses B<AUTOLOAD> to translate calls to undefined methods
on a wine object into calls to the wine API function of the same
name.  The practical upshot is that if you have a wine object named
B<$wine>, and you call B<$wine-E<gt>SomeWineFunction(arg1, arg2,
...)>, it gets translated into B<$wine>->call(), with B<function>
set to SomeWineFunction(), and B<args> set to [ arg1, arg2,
... ] (note that the anonymous array constructor gets supplied
automatically, so you don't need to specify it).

Thus, if you set B<module> and B<ret_type> ahead of time with
accessor functions, you can call a wine API function as though
it were an object method and pass args to it directly.

For example:

    $wine->module ("kernel32");
    $wine->ret_type ("int");

    $atom = $wine->GlobalAddAtomA ("foo");
    $atom = $wine->GlobalAddAtomA ("bar");
    $atom = $wine->GlobalAddAtomA ("xyz");

calls API function GlobalAddAtomA(), passing "foo", "bar", and
"xyz" respectively.

Similarly:

    $wine->module ("kernel32");
    $wine->ret_type ("int");

    $buf = "." x 20;
    $len = $wine->GlobalGetAtomNameA ($atom, \$buf, length ($buf));

calls GlobalGetAtomNameA(), passing $atom, $buf (by reference),
and length($buf), respectively.  Note that the argumentss to
the method are considered to consist entirely of args to pass
through to GlobalGetAtomNameA(), so there is no need for the
hash-initializer syntax, the B<args> key, or the anonymous array
constructor (probably the biggest advantage to doing it that way).


=head2 Chain modifier functions

A set of modifier functions is defined which provides a way to
set values to be passed to the call() method for the duration of
that call only.  Each function returns the address of the object
it operated on, so they can be chained together with Perl's ->
operator.  The currently defined modifier functions are:

S< >S< >S< >S< >B<kernel32>     Set B<module> to B<kernel32>

S< >S< >S< >S< >B<user32>       Set B<module> to B<user32>

S< >S< >S< >S< >B<int>          Set B<ret_type> to B<int>

S< >S< >S< >S< >B<ptr>          Set B<ret_type> to B<ptr>

S< >S< >S< >S< >B<debug_on>     Turn B<debug> on

S< >S< >S< >S< >B<debug_off>    Turn B<debug> off

For example:

    $atom = $wine->int->kernel32->call
    (
        function => 'GlobalAddAtomA',
        args     => 'xyz'
    );

sets B<ret_type> to "int", B<module> to "kernel32", and then
calls GlobalAddAtomA().

The following example combines the B<AUTOLOAD> method and chain
modifier features together, and does the same thing:

    $atom = $wine->int->kernel32->GlobalAddAtomA ("xyz");

The chain modifier functions set values for the duration of
that call only, so in both cases, when the call is done, the
B<ret_type> and B<module> keys of the $wine object will be back
to what they were before the call.  To set them permanently,
use the accessor functions:

    $wine->module   ("kernel32");
    $wine->ret_type ("int");

    # module is kernel32 by default
    $atom = $wine->GlobalAddAtomA ("foo");

    # module is user32, for this call only
    $wine->user32->MessageBoxA (0, "foo", "bar", 4);

    # module is still kernel32 by default
    $atom = $wine->GlobalAddAtomA ("bar");



=head1 $wine::err VARIABLE

Within wine, an integer error code is maintained which always
contains the status of the last API function called.  In wine code,
it is accessed via the GetLastError() function.  From a wine object,
it can be accessed via the package global $wine::err.  For example:

    $wine->int->kernel32->GlobalGetAtomNameA ($atom, \$buf, -1);
    if ($wine::err == 234)
    {
        ...
    }

Wine returns 234 (ERROR_MORE_DATA) from the GlobalGetAtomNameA()
API function in this case because the buffer length passed is -1
(hardly enough room to store anything in ...)



=head1 UTILITIES MODULE

The bundle that includes the wine extension also includes a module of
plain ol' Perl subroutines which are useful for interacting with wine
API functions.  To access them:

    use wine_util;

Currently supported functions are:

=over 4

=item hd BUF [, LENGTH]

Dump a formatted hex dump to STDOUT.  BUF is a string containing
the buffer to dump; LENGTH is the length to dump (length (BUF) if
omitted).  This is handy because wine often writes a null character
into the middle of a buffer, thinking that the next piece of code to
look at the buffer will be a piece of C code that will regard it as
a string terminator.  Little does it know that the buffer is going
to be returned to a Perl script, which may not ...

=item wc STR

Generate and return a wide-character (Unicode) string from the given
ASCII string

=item wclen WSTR

Return the length of the given wide-character string

=item assert CONDITION

Print a message if the assertion fails (i.e., CONDITION is false),
or do nothing quietly if it is true.  The message includes the script
name and line number of the assertion that failed.

=back



=head1 AUTHOR

John F Sturtz, jsturtz@codeweavers.com

=head1 SEE ALSO

wine documentation

=cut



    # ----------------------------------------------------------------
    # | Globals                                                      |
    # |                                                              |
    # |                                                              |
    # ----------------------------------------------------------------
    $err = 0;  # Set to GetLastError() from wine lib



    # ----------------------------------------------------------------
    # | Methods/Subs                                                 |
    # |                                                              |
    # |                                                              |
    # ----------------------------------------------------------------



# ------------------------------------------------------------------------
# | Method:    new                                                       |
# | -------------------------------------------------------------------- |
# | Purpose:   Object constructor                                        |
# |                                                                      |
# | Usage:     $obj->new                                                 |
# |                                                                      |
# | Returns:   new wine object                                           |
# ------------------------------------------------------------------------
sub new
{
    # ----------------------------------------------
    # | Locals                                     |
    # ----------------------------------------------
    my  $pgk      = shift;

    my  $self;

# Begin new

    # --------------------------------------------------------------
    # | Create object                                              |
    # --------------------------------------------------------------
    $self =
    {
        callback => undef,
        module   => "kernel32",
        function => "lstrlenA",
        ret_type => "int",
        args     => [ "dogblaster" ],
        debug    => 0,
        printed  => 0
    };

    # --------------------------------------------------------------
    # | Bless it (my son)                                          |
    # --------------------------------------------------------------
    bless $self, $pkg;

} # End new



# ------------------------------------------------------------------------
# | Method:    init                                                      |
# | -------------------------------------------------------------------- |
# | Purpose:   Initialize wine                                           |
# |                                                                      |
# | Usage:     $obj->init                                                |
# |            $obj->init CALLBACK                                       |
# |                                                                      |
# |     CALLBACK specifies the function that will get control            |
# |     after wine is initialized; if omitted, the default is            |
# |     wine_callback()                                                  |
# |                                                                      |
# | Returns:   Nothing.                                                  |
# |                                                                      |
# | Not only does this method not return anything, it doesn't return     |
# | at all.  It does wine initialization, which in turn transfers con-   |
# | trol to a callback routine (this is dictated by the way the wine     |
# | initialization routines are written).                                |
# |                                                                      |
# | For example, suppose the user has written a Perl script called       |
# | foo.pl, which looks like this:                                       |
# |                                                                      |
# |     foo.pl                                                           |
# |     ------                                                           |
# |                                                                      |
# |     $obj = wine->new ("run_wine_tests");                             |
# |     $obj->init ();                                                   |
# |                                                                      |
# |     sub run_wine_tests                                               |
# |     {                                                                |
# |         <code that makes wine API calls>                             |
# |     }                                                                |
# |                                                                      |
# | When $obj->init() is called in foo.pl, control proceeds through      |
# | several modules, as follows:                                         |
# |                                                                      |
# |    ------------       -----------       ---------------------        |
# |   | wine.pm    |     | wine.xs   |     | wine_perl_gateway.c |       |
# |    ------------       -----------       ---------------------        |
# |   | $obj->init | --> | wine_init | --> | perl_init_wine      | ---   |
# |    ------------       -----------       ---------------------     |  |
# |                                                                   |  |
# |                                                                   |  |
# |              --------------------      ---------------------      |  |
# |             | wine.xs            |     | (wine share obj)    |    |  |
# |              --------------------      ---------------------      |  |
# |    -------- | wine_init_callback | <-- | PROCESS_InitWinelib | <--   |
# |   |          --------------------      ---------------------         |
# |   |                                                                  |
# |   |                                                                  |
# |   |     ----------------                                             |
# |   |    | foo.pl         |                                            |
# |   |     ----------------                                             |
# |    --> | run_wine_tests |                                            |
# |         ----------------                                             |
# |                                                                      |
# | The name of the callback (run_wine_tests(), in this case) is passed  |
# | all the way through each function.  Function wine_init_callback() is |
# | defined in a .spec file as the entry point after wine is initial-    |
# | ized.  So when PROCESS_InitWinelib() is called, wine_init_callback() |
# | gets control, and then it calls run_wine_tests().                    |
# ------------------------------------------------------------------------
sub init
{
    # ----------------------------------------------
    # | Locals                                     |
    # ----------------------------------------------
    my  $self = shift;

# Begin init

    # --------------------------------------------------------------
    # | Righty then; call the wine initialization routine          |
    # --------------------------------------------------------------
    $self->{callback} = shift || "wine_callback";
    wine_init ("main::" . $self->{callback});


    # --------------------------------------------------------------
    # | Bye-bye -- we'll never get here                            |
    # --------------------------------------------------------------
    printf STDERR "Yikes!  How did I get here???\n";

} # End init



# ------------------------------------------------------------------------
# | Method:      call                                                    |
# | -------------------------------------------------------------------- |
# | Purpose:     Call a wine API function                                |
# |                                                                      |
# | Usage:       call ARGS                                               |
# |                                                                      |
# |     where ARGS is a hash initializer with the following format:      |
# |                                                                      |
# |     (                                                                |
# |         module   => <module_name>,                                   |
# |         function => <function_name>,                                 |
# |         ret_type => <int|ptr>,                                       |
# |         args     => [ <arg1>, <arg2>, ... ]                          |
# |     )                                                                |
# |                                                                      |
# | Returns:     value returned by API function called                   |
# ------------------------------------------------------------------------
sub call
{
    # ----------------------------------------------
    # | Locals                                     |
    # ----------------------------------------------
    my  $self = shift;

    my  %a =
    (
        module   => $self->{module},
        function => $self->{function},
        ret_type => $self->{ret_type},
        args     => $self->{args},
        debug    => $self->{debug}
    );

    my  ($module, $function, $ret_type);
    my  $z;

    my  $r;

# Begin call

    # --------------------------------------------------------------
    # | Return-type constants                                      |
    # |                                                            |
    # | [todo]  I think there's a way to define these in a C       |
    # |         header file, so that both the C functions in the   |
    # |         XS module and the Perl routines in the .pm have    |
    # |         access to them.  But I haven't worked it out       |
    # |         yet ...                                            |
    # |                                                            |
    # | [todo]  Support a void type?                               |
    # --------------------------------------------------------------
    my  $RET_INT = 0;
    my  $RET_PTR = 1;

    # --------------------------------------------------------------
    # | For convenience, some extra argument processing:           |
    # |                                                            |
    # | If a single argument is passed, assume we use that as the  |
    # | value for the 'args' key; other keys obtain default values |
    # | from the object.                                           |
    # |                                                            |
    # | Then, as a final touch, if the value of the 'args' key is  |
    # | a scalar, convert it to an anonymous array.                |
    # --------------------------------------------------------------
    if (@_ == 1)
    {
        $a{args} = shift;
    }
    else
    {
        %a = (%a, @_);
    }
    if (! ref ($a{args}))
    {
        $a{args} = [ $a{args} ];
    }

    # --------------------------------------------------------------
    # | Nab module and function names                              |
    # --------------------------------------------------------------
    $module   = $a{module};
    $function = $a{function};

    # --------------------------------------------------------------
    # | Convert return type to an integer                          |
    # --------------------------------------------------------------
    for ($a{ret_type})
    {
        if    ($a{ret_type} eq "int") { $ret_type = $RET_INT }
        elsif ($a{ret_type} eq "ptr") { $ret_type = $RET_PTR }
    } 

    # --------------------------------------------------------------
    # | Debug                                                      |
    # --------------------------------------------------------------
    if ($a{debug})
    {
        print "\n"
            if ($self->{printed});
        $self->{printed} = 1;
        $z = "[$module.$function() / " . scalar (@{$a{args}}) . " arg(s)]";
        print "=== $z ", ("=" x (56 - length ($z))), "\n";
        print "    [wine.pm/obj->call()]\n";
        for (@{$a{args}})
        {
            print "        ", +(ref () ? ("(" . ${$_} . ")") : "$_"), "\n";
        }
    }

    # --------------------------------------------------------------
    # | Now call call_wine_API(), which will turn around and call  |
    # | the appropriate wine API function.  Arguments to           |
    # | call_wine_API() are:                                       |
    # |                                                            |
    # |     module_name                                            |
    # |     function_name                                          |
    # |     return_type                                            |
    # |     debug_flag                                             |
    # |     [args to pass through to wine API function]            |
    # --------------------------------------------------------------
    $r = call_wine_API
    (
        $module,
        $function,
        $ret_type,
        $a{debug},
        @{$a{args}}
    );


    # --------------------------------------------------------------
    # | Debug                                                      |
    # --------------------------------------------------------------
    if ($a{debug})
    {
        $z = "[$module.$function()] -> [$r]";
        print "=== $z ", ("=" x (56 - length ($z))), "\n";
    }


    # --------------------------------------------------------------
    # | Pass the return value back                                 |
    # --------------------------------------------------------------
    return ($r);

} # End call
