# --------------------------------------------------------------------------------
# | Module:      wine_util.pm                                                    |
# | ---------------------------------------------------------------------------- |
# | Purpose:                                                                     |
# |                                                                              |
# | Methods:                                                                     |
# |                                                                              |
# --------------------------------------------------------------------------------
package wine_util;

    use strict;
    use vars qw(@ISA @EXPORT @EXPORT_OK);

    require Exporter;

    @ISA = qw(Exporter);

    @EXPORT = qw(
        hd
        wc
        wclen
        assert
    );



    # ----------------------------------------------------------------
    # | Subs                                                         |
    # |                                                              |
    # |                                                              |
    # ----------------------------------------------------------------



    # ----------------------------------------------------------------------
    # | Subroutine:  hd                                                    |
    # |                                                                    |
    # | Purpose:     Display a hex dump of a string                        |
    # |                                                                    |
    # | Usage:       hd STR                                                |
    # | Usage:       hd STR, LENGTH                                        |
    # |                                                                    |
    # | Returns:     (none)                                                |
    # ----------------------------------------------------------------------
    sub hd
    {
        # Locals
        my  ($buf, $length);
        my  $first;
        my  ($str1, $str2, $str, $t);
        my  ($c, $x);
    
    # Begin sub hd
    
        # --------------------------------------------------------------
        # | Get args; if no BUF specified, blow                        |
        # --------------------------------------------------------------
        $buf = shift;
        $length = (shift or length ($buf));
        return
            if ((not defined ($buf)) || ($length <= 0));

        # --------------------------------------------------------------
        # | Initialize                                                 |
        # --------------------------------------------------------------
        $first = 1;
        $str1 = "00000:";
        $str2 = "";

        # --------------------------------------------------------------
        # | For each character                                         |
        # --------------------------------------------------------------
        for (0 .. ($length - 1))
        {
            $c = substr ($buf, $_, 1);
            $x = sprintf ("%02x", ord ($c));
            $str1 .= (" " . $x);
            $str2 .= (((ord ($c) >= 33) && (ord ($c) <= 126)) ? $c : ".");

            # --------------------------------------------------------------
            # | Every group of 4, add an extra space                       |
            # --------------------------------------------------------------
            if
            (
                ((($_ + 1) % 16) == 4)  ||
                ((($_ + 1) % 16) == 12)
            )
            {
                $str1 .= " ";
                $str2 .= " ";
            }

            # --------------------------------------------------------------
            # | Every group of 8, add a '-'                                |
            # --------------------------------------------------------------
            elsif
            (
                ((($_ + 1) % 16) == 8)
            )
            {
                $str1 .= " -";
                $str2 .= " ";
            }

            # --------------------------------------------------------------
            # | Every group of 16, dump                                    |
            # --------------------------------------------------------------
            if
            (
                ((($_ + 1) % 16) == 0)      ||
                ($_ == ($length - 1))
            )
            {
                $str = sprintf ("%-64s%s", $str1, $str2);
                if ($first)
                {
                    $t = ("-" x length ($str));
                    print "  $t\n";
                    print "  | $length bytes\n";
                    print "  $t\n";
                    $first = 0;
                }
                print "  $str\n";
                $str1 = sprintf ("%05d:", ($_ + 1));
                $str2 = "";
                if ($_ == ($length - 1))
                {
                    print "  $t\n";
                }
            }

        }  # end for


        # --------------------------------------------------------------
        # | Exit point                                                 |
        # --------------------------------------------------------------
        return;

    } # End sub hd



    # ----------------------------------------------------------------------
    # | Subroutine:  wc                                                    |
    # |                                                                    |
    # | Purpose:     Generate unicode string                               |
    # |                                                                    |
    # | Usage:       wc ASCII_STRING                                       |
    # |                                                                    |
    # | Returns:     string generated                                      |
    # ----------------------------------------------------------------------
    sub wc
    {
        # Locals

        return (join ("\x00", split (//, (shift))) . "\x00" . "\x00\x00");

    } # End sub wc



    # ----------------------------------------------------------------------
    # | Subroutine:  wclen                                                 |
    # |                                                                    |
    # | Purpose:     Return length of unicode string                       |
    # |                                                                    |
    # | Usage:       wclen UNICODE_STRING                                  |
    # |                                                                    |
    # | Returns:     string generated                                      |
    # ----------------------------------------------------------------------
    sub wclen
    {
        # Locals
        my  $str = shift;
        my  ($c1, $c2, $n);
    
    # Begin sub wclen

        $n = 0;
        while (length ($str) > 0)
        {
            $c1 = substr ($str, 0, 1, "");
            $c2 = substr ($str, 0, 1, "");
            (($c1 eq "\x00") && ($c2 eq "\x00")) ? last : $n++;
        }

        return ($n);

    } # End sub wclen



    # ----------------------------------------------------------------------
    # | Subroutine:  assert                                                |
    # |                                                                    |
    # | Purpose:     Print warning if something fails                      |
    # |                                                                    |
    # | Usage:       assert CONDITION                                      |
    # |                                                                    |
    # | Returns:     (none)                                                |
    # ----------------------------------------------------------------------
    sub assert
    {
        # Locals
        my  $assertion = shift;
        my  ($fn, $line);
    
    # Begin sub assert

        ($fn, $line) = (caller (0))[1,2];
        ($assertion) or print "Assertion failed [$fn, line $line]\n";

    } # End sub assert



# --------------------------------------------------------------------------------
# | End module                                                                   |
# --------------------------------------------------------------------------------
1;
